/*      Author: Miroslav Skrbek, Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda implementuje Savitzky-Golay filter.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Miroslav Skrbek, Jan Dolezel
 * @version 1.2.0
 */
public class SavitzkyGolayFilter {
  private SavitzkyGolayFilter() {}

  /**
   * vrac jeden dek filtru pro zadan okno a d
   * @param window int velikost okna filtru(mlo by bt lich slo)
   * @param M int d filtru
   * @return Vektor
   */
  public static Vektor getVector(int window, int M) {
    return getVector(window / 2, window / 2, M);
  }

  /**
   * vrac jeden dek filtru pro zadan okno a d
   * @param nl int poet vzork nalevo od stedu filtru
   * @param nr int poet vzork napravo od stedu filtru
   * @param M int d filtru
   * @return Vektor
   */
  public static Vektor getVector(int nl, int nr, int M) {
    int length = nl + nr + 1;
    Vektor res = new Vektor(length);

    Matrix a = new Matrix(M + 1);
    for (int i = 0; i <= M; i++) {
      for (int j = 0; j <= M; j++) {
        double s = 0;
        for (int k = -nl; k <= nr; k++) {
          s += Math.pow(k, i + j);
        }
        a.set(i, j, s);
      }
    }

    Matrix ai = a.inverse();

    for (int i = -nl; i <= nr; i++) {
      double s = 0;
      for (int j = 0; j <= M; j++) {
        s += ai.get(0, j) * Math.pow(i, j);
      }
      res.set(i + nl, s);
    }

    return res;
  }

  /**
   * vrac matici filtru pro zadan okno a d
   * @param window int velikost okna (mlo by bt lich slo)
   * @param M int d filtru
   * @return Matrix
   */
  public static Matrix getMatrix(int window, int M) {
    return getMatrix(window / 2, window / 2, M);
  }

  /**
   * vrac matici filtru pro zadan okno a d
   * @param nl int poet vzork nalevo od stedu filtru
   * @param nr int poet vzork napravo od stedu filtru
   * @param M int d filtru
   * @return Matrix
   */
  public static Matrix getMatrix(int nl, int nr, int M) {
    int len = nl + nr + 1;
    Matrix m = new Matrix(len);
    for (int i = 0; i < len; i++) {
      Vektor v = getVector(i, nl + nr - i, M);
      for (int j = 0; j < len; j++) {
        m.set(j, i, v.get(j));
      }
    }
    return m;
  }

  /**
   * filtruje posloupnost relnch sel pro zadan okno a d
   * @param row double[] vstupn data
   * @param nl int poet vzork nalevo od stedu filtru
   * @param nr int poet vzork napravo od stedu filtru
   * @param M int d filtru
   * @return double[]
   */
  public static double[] filter(double[] row, int nl, int nr, int M) {
    return Filter.filter(row, getMatrix(nl, nr, M), nl);
  }

  /**
   * filtruje posloupnost relnch sel pro zadan okno a d
   * @param row double[] vstupn data
   * @param M int d filtru
   * @param window int velikost okna (mlo by bt lich slo)
   * @return double[]
   */
  public static double[] filter(double[] row, int M, int window) {
    return filter(row, window / 2, window / 2, M);
  }

  /**
   * Vypot vektor s adaptivn dlkou pro danou velikost vektoru a maximln dlku
   * @param size int velikost vektrou
   * @param l int maximln dlka
   * @return int[]
   */
  private static int[] getLength(int size, int l) {
    int len[] = new int[size];
    int brk1 = size / 5;
    int brk2 = size - size / 40;
    if(l > size) l = size;
    int start = 2 * (l / 12) + 1;
    int top = 2 * (l / 2) + 1;
    int stop = 2 * (l / 4) + 1;
    double k1 = (top - start) / (double) brk1;
    double k2 = (stop - top) / (double) (size - brk2);

    for (int i = 0; i < brk1; i++) {
      len[i] = 2 * ( (int) (start + i * k1) / 2) + 1;
    }
    for (int i = brk1; i < brk2; i++) {
      len[i] = top;
    }
    for (int i = brk2; i < size; i++) {
      len[i] = 2 * ( (int) (top + (i - brk2 + 1) * k2) / 2) + 1;
    }

    return len;
  }

  /**
   * Adaptivn (s promnou dlkou filtru) filtruje data
   * @param data double[] vstupn vektor
   * @param M int d filtru
   * @param len int maximln velikost okna filtru
   * @return double[]
   */
  public static double[] filterAdaptive(double[] data, int M, int len) {
    int size = data.length;
    double res[] = new double[size];
    int length[] = getLength(size, len);
    int lastnl = 0;
    int lastnr = 0;
    Vektor sg = null;

    for (int i = 0; i < size; i++) {
      int nl = (length[i] - 1) / 2;
      int start = Math.max(0, Math.min(i - nl, size - length[i]));
      if (i < nl) {
        nl = i;
      }
      else if ( (i + nl) > size) {
        nl = length[i] - (size - i);
      }

      int nr = length[i] - nl - 1;
      if ( (nl != lastnl) || (nr != lastnr)) {
        sg = getVector(nl, nr, M);
      }
      double c = 0;
      for (int j = 0; j < length[i]; j++) {
        c += sg.get(j) * data[start + j];
      }
      res[i] = c;
      lastnl = nl;
      lastnr = nr;
    }
    return res;
  }
}
