/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import spirals.motol.loaders.XML.XMLDatabaseLoader;
import spirals.motol.loaders.DatabaseLoader;
import tablet.TabletInfo;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

/**
 * <p>Class contains information about application configuration and settings</p>
 * <p>All information are saved in XML file</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class Configuration {
  /**
   * Tda, kter nahrv databzi
   */
  private static DatabaseLoader databaseLoader = null;
  /**
   * list of known tablets
   */
  private List<TabletInfo> tablets;
  /**
   * list of available types of drawings
   */
  private List<Test> tests;
  /**
   * index of selected tablet in tablet list
   */
  private int selectedTabletIndex = -1;
  /**
   * file with configuration
   */
  private File config = new File("config.xml");
  /**
   * database directory
   */
  private File database;
  /**
   * look and feel class for UI Manager
   */
  private String lafClass;
  /**
   * maximln as nahrvn z tabletu
   */
  private int timeout;
  /**
   * XML parser
   */
  public static final SAXBuilder builder = new SAXBuilder(
      "org.apache.xerces.parsers.SAXParser");
  /**
   * XML outputter
   */
  public static final XMLOutputter serializer = new XMLOutputter();

  /**
   * name of configuration tag (root element)
   */
  public static final String configurationTag = "configuration";
  /**
   * name of tablet tag
   */
  public static final String tabletTag = "tablet";
  /**
   * name of tablet name attribute
   */
  public static final String tabletNameAttr = "name";
  /**
   * name of tabet's minimal x coordinate attribute
   */
  public static final String tabletMinXAttr = "minx";
  /**
   * name of tabet's maximal x coordinate attribute
   */
  public static final String tabletMaxXAttr = "maxx";
  /**
   * name of tabet's minimal y coordinate attribute
   */
  public static final String tabletMinYAttr = "miny";
  /**
   * name of tabet's maximal y coordinate attribute
   */
  public static final String tabletMaxYAttr = "maxy";
  /**
   * name of default tablet tag
   */
  public static final String defaultTabletTag = "defTablet";
  /**
   * name of default tablet tag attribute
   */
  public static final String defTabletValueAttr = "value";
  /**
   * name of type tag
   */
  public static final String typeTag = "type";
  /**
   * name of timeout tag
   */
  public static final String timeoutTag = "timeout";
  /**
   * name of timeout tag attribute
   */
  public static final String timeoutValueAttr = "value";
  /**
   * name of database root tag
   */
  public static final String databaseRootTag = "databaseRoot";
  /**
   * name of database root tag attribute
   */
  public static final String databaseRootValueAttr = "value";
  /**
   * name of look and feel class tag
   */
  public static final String lafClassTag = "lafClass";

  /**
   * Constructor
   * <p>Method loads configuration form file config.xml</p>
   */
  public Configuration() {
    serializer.setFormat(org.jdom.output.Format.getPrettyFormat());
    try {
      Document doc = builder.build(config);
      Element root = doc.getRootElement();
      // read known tablet measures
      List<Element> tabs = root.getChildren(tabletTag);
      tablets = new ArrayList<TabletInfo> (tabs.size());
      for (int i = 0; i < tabs.size(); i++) {
        String name = tabs.get(i).getAttributeValue(tabletNameAttr);
        int minx = Integer.parseInt(tabs.get(i).getAttributeValue(
            tabletMinXAttr));
        int maxx = Integer.parseInt(tabs.get(i).getAttributeValue(
            tabletMaxXAttr));
        int miny = Integer.parseInt(tabs.get(i).getAttributeValue(
            tabletMinYAttr));
        int maxy = Integer.parseInt(tabs.get(i).getAttributeValue(
            tabletMaxYAttr));
        tablets.add(new TabletInfo(name, minx, maxx, miny, maxy));
      }
      // read selected tablet
      selectedTabletIndex = Integer.parseInt(root.getChild(defaultTabletTag).
                                             getAttributeValue(
          defTabletValueAttr));
      // read available types for drawings
      List<Element> strTypes = root.getChildren(typeTag);
      tests = new ArrayList<Test> (strTypes.size());
      for (Element e : strTypes) {
        tests.add(new Test(e));
      }
      // read look and feel class
      lafClass = root.getChildText(lafClassTag);
      timeout = Integer.parseInt(root.getChildText(timeoutTag));
      database = new File(root.getChildText(databaseRootTag));
    }
    catch (JDOMException e) {
      System.err.println("Chyba parsovani souboru databaze.");
      System.err.println(e.getMessage());
      System.exit(1);
    }
    catch (IOException e) {
      System.err.println(e);
      System.exit(1);
    }
  }

  /**
   * save configuration to file
   */
  public void save() {
    Element root = new Element(configurationTag);
    // write tablet measures
    for (TabletInfo ti : tablets) {
      Element tab = new Element(tabletTag);
      tab.setAttribute(tabletNameAttr, ti.getTabletName());
      tab.setAttribute(tabletMinXAttr, "" + ti.getMinX());
      tab.setAttribute(tabletMaxXAttr, "" + ti.getMaxX());
      tab.setAttribute(tabletMinYAttr, "" + ti.getMinY());
      tab.setAttribute(tabletMaxYAttr, "" + ti.getMaxY());
      root.addContent(tab);
    }
    // write default tablet index
    Element defTab = new Element(defaultTabletTag);
    defTab.setAttribute(defTabletValueAttr, "" + selectedTabletIndex);
    root.addContent(defTab);
    // write available types for drawings
    for (Test test : tests) {
      root.addContent(test.toElement());
    }
    // write look and feel class
    Element laf = new Element(lafClassTag);
    laf.setText(lafClass);
    root.addContent(laf);
    // write timeout value
    Element to = new Element(timeoutTag);
    to.setText("" + timeout);
    root.addContent(to);
    // write root of database directory in XML
    Element dr = new Element(databaseRootTag);
    dr.setText(database.getAbsolutePath());
    root.addContent(dr);
    // write to disk
    Document doc = new Document(root);
    OutputStream os = null;
    try {
      os = new FileOutputStream(config);
      serializer.output(doc, os);
      os.close();
    }
    catch (IOException e) {
      System.err.println(e);
    }
    finally {
      if (os != null) {
        try {
          os.close();
        }
        catch (IOException e) {}
      }
    }
  }

  /**
   * returns list of known tablets
   */
  public List<TabletInfo> getAvailableTablets() {
    return tablets;
  }

  /**
   * returns index of selected tablet in list of tablets
   */
  public int getSelectedTabletIndex() {
    return selectedTabletIndex;
  }

  /**
   * sets index of new selected tablet
   */
  public void setSelectedTabletIndex(int index) {
    selectedTabletIndex = index;
  }

  /**
   * returns directory where database is saved
   */
  public File getDatabaseRoot() {
    return database;
  }

  /**
   * returns list of available types of drawings
   */
  public List<Test> getTests() {
    return tests;
  }

  /**
   * set new look and feel class
   */
  public void setLafClass(String lafClass) {
    this.lafClass = lafClass;
  }

  /**
   * returns string representation of current look and feel class
   */
  public String getLafClass() {
    return lafClass;
  }

  /**
   * @return int poet vtein, kdy tablet sbr data
   */
  public int getTimeout() {
    return timeout;
  }

  /**
   * vrac implicitn tdu pro nahrvn dat
   * @return DatabaseLoader
   */
  public DatabaseLoader getDatabaseLoader() {
    if (databaseLoader == null) {
      databaseLoader = new XMLDatabaseLoader(database);
      ( (XMLDatabaseLoader) databaseLoader).defaultRootFile = this.database;
    }
    return databaseLoader;
  }
}
