/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import spirals.motol.loaders.*;
import java.util.*;

/**
 * <p>Root class for storing database.</p>
 * <p>Each time database is exported all pacients are stored in random order without
 * surname and firstname. No corespondence between database and exported data should
 * be found.</p>
 * <p>Database contains Pacients. Pacients have Visits and Visits consist of Data.
 * Database is stored in directory 'data' containing root file and direcotries of pacients.
 * Only root file is loaded on application startup. Each pacient directory contains
 * xml files representing visits and zip archives with captured tablet data for visits.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @see spirals.data.TabletData TabletData
 * @see Data Data
 * @see Pacient Pacient
 * @see Visit Visit
 *
 * @author Jan Dolezel
 * @version 1.1.2
 */
public class Database {
  /**
   * rozcestnk po tdch
   */
  private Engine engine;
  /**
   * tda pro nahrn databze
   */
  private DatabaseLoader loader;
  /**
   * hash tabulka pacient jednoznan identifikovanch rodnm slem
   */
  private HashMap<Long, Pacient> database = null;
  /**
   * seznam vech pacient
   */
  private List<Pacient> listAll = null;
  /**
   * seznam vybranch pacient z vyhledvn
   */
  private List<Pacient> listRestricted = null;
  /**
   * pouvat seznam vyhledvn
   */
  private boolean restricted = false;
  /**
   * zpsob azen pacient
   */
  private SORT howSort = SORT.byName;
  /**
   * datum psledn zlohy databze
   */
  private Date lastBackup = null;

  public Database(DatabaseLoader loader) {
    this.loader = loader;
    listAll = new ArrayList<Pacient> (0);
    database = new HashMap<Long, Pacient> (0);
    lastBackup = new Date();
    listRestricted = new ArrayList<Pacient> (0);
  }

  public Database(Engine engine) {
    this(engine, engine.getConfiguration().getDatabaseLoader());
  }

  public Database(Engine engine, DatabaseLoader loader) {
    this(loader);
    this.engine = engine;
    loader.load(this);
    this.sort();
  }

  public void save() {
    loader.save(this);
  }

  public List<Pacient> getAll() {
    return listAll;
  }

  public List<Pacient> getRestricted() {
    return listRestricted;
  }

  /**
   * sma seznam vyhledanch pacient
   */
  public void clearRestricted() {
    listRestricted = new ArrayList();
  }

  /**
   * pidej do seznamu vyhledvn pacienty s nvtvou mezi zadanmi daty
   * @param dateBegin Date
   * @param dateEnd Date
   */
  public void setRestrictedVisit(Date dateBegin, Date dateEnd) {
    for (Pacient p : database.values()) {
      if (p.getLastVisit() != null) {
        if (p.getLastVisit().getVisitDate().compareTo(dateBegin) >= 0 &&
            p.getLastVisit().getVisitDate().compareTo(dateEnd) <= 0 &&
            !listRestricted.contains(p)) {
          listRestricted.add(p);
        }
      }
    }
  }

  /**
   * pidej do seznamu vyhledvn pacienty s datem narozen mezi zadanmi daty
   * @param dateBegin Date
   * @param dateEnd Date
   */
  public void setRestrictedBorn(Date dateBegin, Date dateEnd) {
    for (Pacient p : database.values()) {
      if (p.getBornDate().compareTo(dateBegin) >= 0 &&
          p.getBornDate().compareTo(dateEnd) <= 0 && !listRestricted.contains(p)) {
        listRestricted.add(p);
      }
    }
  }

  /**
   * pidej do seznamu vyhledvn pacienty s pjmenm v danm rozsahu
   * @param nameBegin String
   * @param nameEnd String
   */
  public void setRestrictedName(String nameBegin, String nameEnd) {
    for (Pacient p : database.values()) {
      if ( (Engine.AbcComparator.getInstance().compare(p.getSurname(),
          nameBegin) >= 0) &&
          (Engine.AbcComparator.getInstance().compare(p.getSurname(), nameEnd) <=
           0) && !listRestricted.contains(p)) {
        listRestricted.add(p);
      }
    }
  }

  /**
   * Pidej do seznamu vyhledvn pacienta s danm ID
   * @param ID String
   */
  public void setRestrictedID(String ID) {
    for (Pacient p : database.values()) {
      if (p.getID().equals(ID)) {
        listRestricted.add(p);
      }
    }
  }

  public void setRestricted(boolean restricted) {
    this.restricted = restricted;
    this.sort();
  }

  /**
   * poet zobrazovanch pacient
   * @return int
   */
  public int getTreeCount() {
    if (restricted) {
      return listRestricted.size();
    }
    else {
      return listAll.size();
    }
  }

  /**
   * vrat pacienta na dan pozici v zobrazovanm strom
   * @param index int
   * @return Pacient
   */
  public Pacient getTreePacient(int index) {
    if (restricted) {
      return listRestricted.get(index);
    }
    else {
      return listAll.get(index);
    }
  }

  /**
   * vrat pozici pacienta v zobrazovanm strom
   * @param pacient Pacient
   * @return int
   */
  public int getTreeIndex(Pacient pacient) {
    if (restricted) {
      return listRestricted.indexOf(pacient);
    }
    else {
      return listAll.indexOf(pacient);
    }
  }

  public void sort() {
    this.sort(howSort);
  }

  /**
   * sera podle kritria
   * @param how SORT
   */
  public void sort(SORT how) {
    this.howSort = how;
    switch (how) {
      case byName:
        this.sort(Engine.nameComparator);
        break;
      case byBornDate:
        this.sort(Engine.bornComparator);
        break;
      case byLastVisit:
        this.sort(Engine.visitComparator);
        break;
    }
    if ( (engine != null) && (engine.getMainFrame().getTree() != null)) {
      engine.getMainFrame().getTree().getTree().updateUI();
    }
  }

  private void sort(Comparator c) {
    Collections.sort(listAll, c);
    Collections.sort(listRestricted, c);
  }

  /**
   * pidej pacienta, pokud nen v databzi
   * @param pacient Pacient
   */
  public void addPacient(Pacient pacient) {
    long newID = pacient.getLongID();
    if ( (newID != 0) && (!database.containsKey(newID))) {
      database.put(newID, pacient);
      listAll.add(pacient);
    }
    this.sort();
  }

  public long getPacientCount() {
    return database.size();
  }

  public Pacient getPacient(Long ID) {
    return database.get(ID);
  }

  public Date getLastBackup() {
    return lastBackup;
  }

  public Engine getEngine() {
    return engine;
  }

  public DatabaseLoader getLoader() {
    return loader;
  }

  /**
   * vol se, hned po zlohovn
   */
  public void backuped() {
    lastBackup = new Date();
    this.save();
  }

  public void setLastBackup(Date date) {
    this.lastBackup = date;
  }

  /**
   * pokud jet nen databze inicializovan, vyhrad pro ni prostor
   * @param size int
   */
  public void setSize(int size) {
    if (database.size() == 0) {
      listAll = new ArrayList<Pacient> (size);
      database = new HashMap<Long, Pacient> (size);
    }
  }

  public void export(DatabaseLoader exporter) {
    exporter.exportData(this);
  }

  public void importData(DatabaseLoader importer) {
    importer.importData(this);
  }

  public long getDatabaseSize() {
    return loader.getDatabaseSize(this);
  }

  public boolean contains(String ID) {
    return database.containsKey(Long.parseLong(ID));
  }

  public void replace(String oldID, String newID) {
    Pacient p = database.remove(Long.parseLong(oldID));
    if(p != null) database.put(Long.parseLong(newID), p);
  }

  public static enum SORT {
    byName, byBornDate, byLastVisit
  };
}
