/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package tablet;

import java.util.*;
import java.lang.*;
import jwintab.*;

/**
 * <p>Class that communicates with tablet dll library. Accomplish all tablet specific tasks.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Miroslav Skrbek, Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Miroslav Skrbek
 * @author Jan Dolezel
 * @version 1.2
 */
public class Tablet
    implements Runnable {
  private static Tablet instance;
  private static boolean capturing = false;
  private static boolean start = true;
  private static int timeout;

  /**
   * Array of listeners for tablet packets
   */
  private List<TabletListener> listeners = new ArrayList();
  /**
   * Information about opened tablet
   */
  private static TabletInfo ti;
  /**
   * Runs on background and listens to tablet
   */
  private Thread thread = null;
  /**
   * specifies whether is tablet ready for use
   */
  private boolean opened = false;

  /**
   * Creates new tablet with provided information
   * @param tabletInfo tablet measures
   */
  private Tablet(TabletInfo tabletInfo) {
    this.ti = tabletInfo;
  }

  /**
   * Tda Tablet je jedinek. Tovrn metoda
   * @param tabletInfo TabletInfo
   * @param time int
   * @return Tablet
   */
  public static Tablet getInstance(TabletInfo tabletInfo, int time) {
    timeout = time * 1000;
    if ( (instance == null) || (ti != tabletInfo)) {
      instance = new Tablet(tabletInfo);
    }
    return instance;
  }

  /**
   * stops listening to tablet
   */
  public void close() {
    this.stopCapture();
    if (opened) {
      Jwintab.close();
      opened = false;
    }
  }

  /**
   * tries to initialize tablet
   */
  public void open() {
    if (!opened) {
      Jwintab.open();
      opened = true;
    }
  }

  /**
   * loop for handling tablet specific events
   * <p>Method loops until interrupted. When recieve packets from tablets, it
   * converts them to tablet points and let know listeners. First notifing is
   * performed after the tablet pen has touched the surface.</p>
   */
  public void run() {
    int time = 0; // time at start of record
    boolean touched = start; // variable indicates start of record
    int[] packet = new int[8];
    long startTime = System.currentTimeMillis();
    while ( ( (System.currentTimeMillis() - startTime) < 100) &&
           (Jwintab.getPacket(packet) != 0)) {
      ; // clear tablet queue
    }
    startTime = System.currentTimeMillis();
    capturing = true;
    while (capturing) {
      while (Jwintab.getPacket(packet) != 0) {
        // packet is ready
        if (touched) {
          if (packet[TabletPoint.PK_TIME] < time) {
            packet[TabletPoint.PK_TIME] = time - packet[TabletPoint.PK_TIME];
          }
          else {
            packet[TabletPoint.PK_TIME] = packet[TabletPoint.PK_TIME] - time;
          }
          this.notifyListeners(packet);
        }
        else {
          // start of record
          touched = (packet[TabletPoint.PK_BUTTONS] &
                     TabletPoint.BUTTON_TOUCH) != 0;
          if (touched) {
            time = packet[TabletPoint.PK_TIME];
          }
        }
      }
      try {
        thread.sleep(10);
      }
      catch (InterruptedException e) {}
      if ( (System.currentTimeMillis() - startTime) > timeout) {
        this.notifyListenersTimeout();
        return;
      }
    }
  }

  /**
   * starts capturing packets from tablet
   * <p>If tablet is opened, method runs new thread for listening and exits.</p>
   */
  public void startCapture(boolean start) {
    if (opened && !capturing) {
      this.start = start;
      thread = new Thread(this);
      thread.start();
    }
  }

  /**
   * stops capturing packets
   */
  public synchronized void stopCapture() {
    capturing = false;
  }

  /**
   * notify listeners about new tablet packet
   * @param packet array for creating new TabletPoint
   */
  protected synchronized void notifyListeners(int[] packet) {
    int sz = listeners.size();
    TabletPoint tp = new TabletPoint(packet);
    for (int i = 0; i < sz; i++) {
      listeners.get(i).onTabletPacket(tp);
    }
  }

  /**
   * Informuj posluchae, e jsou data sbrna pli dlouho
   */
  protected synchronized void notifyListenersTimeout() {
    int sz = listeners.size();
    for (int i = 0; i < sz; i++) {
      listeners.get(i).onTimeout();
    }
  }

  /**
   * returns information about tablet
   */
  public synchronized TabletInfo getTabletInfo() {
    return ti;
  }

  /**
   * sets new tablet information
   * @param tabletInfo tablet measures
   */
  public synchronized void setTabletInfo(TabletInfo tabletInfo) {
    this.ti = tabletInfo;
  }

  /**
   * adds new TabletListener to list of listeners
   * @param listener class capable process TabletPacket
   */
  public synchronized void addTabletListener(TabletListener listener) {
    if (listener != null) {
      listeners.add(listener);
    }
  }

  /**
   * removes TabletListener from list of listeners
   */
  public synchronized void removeTabletListener(TabletListener listener) {
    listeners.remove(listener);
  }

  /**
   * specifies whether is tablet ready for use
   */
  public synchronized boolean isOpen() {
    return opened;
  }
}
